"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const util_1 = require("util");
// we need to use an import/require here because it does not force consumers to
// enable esModuleInterop in tsconfig.json
const glob = require("glob");
exports.defaultGraphQLProjectName = 'GraphQL';
// temporary utility until `graphql-config` supports this function natively
// see: https://github.com/prisma/graphql-config/pull/113
function resolvePathRelativeToConfig(project, relativePath) {
    return project.resolveConfigPath(relativePath);
}
exports.resolvePathRelativeToConfig = resolvePathRelativeToConfig;
function resolveProjectName(project, defaultName = exports.defaultGraphQLProjectName) {
    return project.projectName || defaultName;
}
exports.resolveProjectName = resolveProjectName;
function resolveSchemaPath(project, ignoreMissing = false) {
    // schemaPath is nullable in graphq-config even though it cannot actually be
    // omitted. This function simplifies access to the schemaPath without
    // requiring a type guard.
    if (!project.schemaPath) {
        // this case should never happen with a properly formatted config file.
        // graphql-config currently does not perform any validation so it's possible
        // for a mal-formed schema to be loaded at runtime.
        throw new Error(`Missing GraphQL schemaPath for project '${resolveProjectName(project)}'`);
    }
    // resolve fully qualified schemaPath
    const schemaPath = project.resolveConfigPath(project.schemaPath);
    if (ignoreMissing) {
        return schemaPath;
    }
    if (!fs_1.existsSync(schemaPath)) {
        const forProject = project.projectName
            ? ` for project '${project.projectName}'`
            : '';
        throw new Error([
            `Schema not found${forProject}.`,
            `Expected to find the schema at '${schemaPath}' but the path does not exist.`,
            `Check '${project.configPath}' and verify that schemaPath is configured correctly${forProject}.`,
        ].join(' '));
    }
    return schemaPath;
}
exports.resolveSchemaPath = resolveSchemaPath;
function getGraphQLProjects(config) {
    const projects = config.getProjects();
    if (projects) {
        // multi-project configuration, return an array of projects
        return Object.values(projects);
    }
    const project = config.getProjectConfig();
    if (project && project.schemaPath) {
        // single project configuration, return an array of the single project
        return [project];
    }
    // invalid project configuration
    throw new Error(`No projects defined in '${config.configPath}'`);
}
exports.getGraphQLProjects = getGraphQLProjects;
function getGraphQLSchemaPaths(config) {
    return getGraphQLProjects(config).reduce((schemas, project) => {
        return schemas.concat(resolveSchemaPath(project));
    }, []);
}
exports.getGraphQLSchemaPaths = getGraphQLSchemaPaths;
function getGraphQLProjectIncludedFilePaths(projectConfig) {
    return __awaiter(this, void 0, void 0, function* () {
        return (yield Promise.all(projectConfig.includes.map((include) => util_1.promisify(glob)(resolvePathRelativeToConfig(projectConfig, include), {
            ignore: projectConfig.excludes.map((exclude) => resolvePathRelativeToConfig(projectConfig, exclude)),
        })))).reduce((allFilePaths, filePaths) => allFilePaths.concat(filePaths), []);
    });
}
exports.getGraphQLProjectIncludedFilePaths = getGraphQLProjectIncludedFilePaths;
function getGraphQLProjectForSchemaPath(config, schemaPath) {
    const project = getGraphQLProjects(config)
        .filter((project) => project.schemaPath === schemaPath)
        .shift() || config.getProjectConfig();
    if (!project || project.schemaPath !== schemaPath) {
        throw new Error(`No project defined in graphql config for schema '${schemaPath}'`);
    }
    return project;
}
exports.getGraphQLProjectForSchemaPath = getGraphQLProjectForSchemaPath;
